package cn.com.qmth.stmms.admin.dto;

import org.apache.commons.lang.StringUtils;

import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.common.annotation.ExcelField;

public class ObjectiveQuestionDTO implements QuestionDTO {

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    @ExcelField(title = "科目代码", align = 2, sort = 10)
    private String subjectCode;

    @ExcelField(title = "科目名称", align = 2, sort = 20)
    private String subjectName;

    @ExcelField(title = "试卷类型", align = 2, sort = 30)
    private String paperType;

    @ExcelField(title = "大题名称", align = 2, sort = 40)
    private String title;

    @ExcelField(title = "大题号(只能用小写数字)", align = 2, sort = 50)
    private Integer mainNumber;

    @ExcelField(title = "小题号(只能用小写数字)", align = 2, sort = 60)
    private Integer subNumber;

    @ExcelField(title = "标准答案", align = 2, sort = 70)
    private String answer;

    @ExcelField(title = "小题满分", align = 2, sort = 80)
    private Double totalScore;

    public ObjectiveQuestionDTO() {

    }

    public ObjectiveQuestionDTO(ExamQuestion question, ExamSubject subject) {
        setSubjectCode(question.getSubjectCode());
        setSubjectName(subject != null ? subject.getName() : "");
        setPaperType(question.getPaperType());
        setMainNumber(question.getMainNumber());
        setSubNumber(question.getSubNumber());
        setTitle(question.getMainTitle());
        setAnswer(question.getAnswer());
        setTotalScore(question.getTotalScore());
    }

    public ExamQuestion transform() {
        ExamQuestion question = new ExamQuestion();
        question.setSubjectCode(StringUtils.trimToNull(subjectCode));
        question.setPaperType(StringUtils.trimToNull(paperType));
        if (question.getPaperType() == null) {
            question.setPaperType(NULL_PAPER_TYPE_PLACEHOLDER);
        }
        question.setMainNumber(mainNumber);
        question.setSubNumber(subNumber);
        question.setGroupNumber(0);
        question.setObjective(true);
        question.setMainTitle(StringUtils.trimToNull(title));
        question.setAnswer(StringUtils.trimToNull(answer));
        question.setTotalScore(totalScore);
        question.setIntervalScore(1d);
        return question;
    }

    public String getSubjectCode() {
        return subjectCode;
    }

    public void setSubjectCode(String subjectCode) {
        this.subjectCode = subjectCode;
    }

    public String getSubjectName() {
        return subjectName;
    }

    public void setSubjectName(String subjectName) {
        this.subjectName = subjectName;
    }

    public Integer getMainNumber() {
        return mainNumber;
    }

    public void setMainNumber(Integer mainNumber) {
        this.mainNumber = mainNumber;
    }

    public Integer getSubNumber() {
        return subNumber;
    }

    public void setSubNumber(Integer subNumber) {
        this.subNumber = subNumber;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getAnswer() {
        return answer;
    }

    public void setAnswer(String answer) {
        this.answer = answer;
    }

    public Double getTotalScore() {
        return totalScore;
    }

    public void setTotalScore(Double totalScore) {
        this.totalScore = totalScore;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }

}
