package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.SubjectiveScore;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.query.SubjectiveScoreSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.SubjectiveScoreService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.mark.service.MarkSpecialTagService;
import cn.com.qmth.stmms.biz.mark.service.MarkTrackService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;
import cn.com.qmth.stmms.common.utils.RequestUtils;

@Controller("inspectedController")
@RequestMapping("/admin/exam/inspected")
public class InspectedController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(InspectedController.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkTrackService markTrackService;

    @Autowired
    private MarkSpecialTagService markSpecialTagService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private FileService fileService;

    @Autowired
    private LockService lockService;

    @Autowired
    private SubjectiveScoreService scoreService;

    @Autowired
    private MarkService markService;

    // 并发处理互斥锁
    private Map<Integer, Integer> currentTaskMap = new HashMap<Integer, Integer>();

    @RequestMapping
    public String list(Model model, HttpServletRequest request, ExamStudentSearchQuery query,
            @RequestParam(required = false) SubjectiveStatus status,
            @RequestParam(required = false) Integer mainNumber, @RequestParam(required = false) Double mainStartScore,
            @RequestParam(required = false) Double mainEndScore, @RequestParam(required = false) Double questionScore) {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        query.setExamId(examId);
        query.setUpload(true);
        query.setAbsent(false);
        query.setBreach(false);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (query.getSubjectCode() == null && !subjectList.isEmpty()) {
            query.setSubjectCode(subjectList.get(0).getCode());
        }
        if (status != null) {
            query.addStatus(status);
        } else {
            query.addStatus(SubjectiveStatus.INSPECTED);
            query.addStatus(SubjectiveStatus.MARKED);
        }
        if (mainNumber != null || mainStartScore != null || mainEndScore != null || questionScore != null) {
            SubjectiveScoreSearchQuery query2 = new SubjectiveScoreSearchQuery();
            query2.setExamId(examId);
            query2.setSubjectCode(query.getSubjectCode());
            query2.setMainNumber(mainNumber);
            query2.setMainStartScore(mainStartScore);
            query2.setMainEndScore(mainEndScore);
            query2.setScore(questionScore);
            query2.setPageSize(Integer.MAX_VALUE);
            List<Integer> studentIds = scoreService.findStudentIdByQuery(query2);
            query.setStudentIds(studentIds);
        }
        if (query.getStudentIds() == null || query.getStudentIds().size() != 0) {
            query = studentService.findByQuery(query);
        }
        model.addAttribute("query", query);
        model.addAttribute("inspectCount", query.getTotalCount());
        model.addAttribute("questionList",
                questionService.findMainByExamAndSubjectAndObjective(examId, query.getSubjectCode(), false));
        model.addAttribute("mainNumber", mainNumber);
        model.addAttribute("mainStartScore", mainStartScore);
        model.addAttribute("mainEndScore", mainEndScore);
        model.addAttribute("questionScore", questionScore);
        model.addAttribute("subjectList", subjectList);
        model.addAttribute("statusList", SubjectiveStatus.getOptionList());
        model.addAttribute("status", status);
        return "modules/exam/inspectedList";
    }

    @Logging(menu = "开始复核", type = LogType.QUERY)
    @RequestMapping("/start")
    @ResponseBody
    public ModelAndView start(HttpServletRequest request, RedirectAttributes redirectAttributes,
            ExamStudentSearchQuery query, @RequestParam(required = false) Integer mainNumber,
            @RequestParam(required = false) Double mainStartScore, @RequestParam(required = false) Double mainEndScore,
            @RequestParam(required = false) Double questionScore) {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        releaseByUser(wu.getUser().getId());
        ModelAndView view = new ModelAndView("modules/exam/inspected");
        query.setExamId(examId);
        query.setUpload(true);
        query.setAbsent(false);
        query.setBreach(false);
        query.addStatus(SubjectiveStatus.MARKED);
        query.setPageSize(Integer.MAX_VALUE);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (query.getSubjectCode() == null && !subjectList.isEmpty()) {
            query.setSubjectCode(subjectList.get(0).getCode());
        }
        if (mainNumber != null || mainStartScore != null || mainEndScore != null || questionScore != null) {
            SubjectiveScoreSearchQuery query2 = new SubjectiveScoreSearchQuery();
            query2.setExamId(examId);
            query2.setSubjectCode(query.getSubjectCode());
            query2.setMainNumber(mainNumber);
            query2.setMainStartScore(mainStartScore);
            query2.setMainEndScore(mainEndScore);
            query2.setScore(questionScore);
            query2.setPageSize(Integer.MAX_VALUE);
            List<Integer> studentIds = scoreService.findStudentIdByQuery(query2);
            query.setStudentIds(studentIds);
        }
        if (query.getStudentIds() == null || query.getStudentIds().size() != 0) {
            query = studentService.findByQuery(query);
        }
        List<Integer> ids = new ArrayList<Integer>();
        if (query.getResult() != null && query.getResult().size() > 0) {
            for (ExamStudent student : query.getResult()) {
                ids.add(student.getId());
            }
        }
        view.addObject("inspectCount", ids.size());
        view.addObject("fileServer", fileService.getFileServer());
        view.addObject("ids", StringUtils.join(ids, ","));
        view.addObject("message", ids.size() > 0 ? "" : "没有待复核的任务");
        return view;
    }

    @Logging(menu = "开始复核", type = LogType.QUERY)
    @RequestMapping("/startById")
    public ModelAndView startById(HttpServletRequest request, RedirectAttributes redirectAttributes,
            @RequestParam Integer studentId) {
        WebUser wu = RequestUtils.getWebUser(request);
        releaseByUser(wu.getUser().getId());
        ModelAndView view = new ModelAndView("modules/exam/inspected");
        List<Integer> ids = new ArrayList<Integer>();
        ExamStudent student = studentService.findById(studentId);
        if (student != null && student.getSubjectiveStatus().equals(SubjectiveStatus.MARKED)) {
            ids.add(studentId);
        }
        view.addObject("inspectCount", ids.size());
        view.addObject("fileServer", fileService.getFileServer());
        view.addObject("ids", StringUtils.join(ids, ","));
        view.addObject("studentId", studentId);
        return view;
    }

    @RequestMapping("/info")
    @ResponseBody
    public Object info(HttpServletRequest request, @RequestParam Integer studentId) {
        WebUser wu = RequestUtils.getWebUser(request);
        ExamStudent student = studentService.findById(studentId);
        JSONObject result = new JSONObject();
        if (setCurrent(student.getId(), wu.getUser().getId())
                && student.getSubjectiveStatus().equals(SubjectiveStatus.MARKED)) {
            result.accumulate("id", student.getId());
            result.accumulate("secretNumber", student.getSecretNumber());
            result.accumulate("subjectCode", student.getSubjectCode());
            result.accumulate("subjectName", student.getSubjectName());
            result.accumulate("totalScore", student.getTotalScore());

            List<MarkGroup> groups = groupService.findByExamAndSubject(student.getExamId(), student.getSubjectCode());
            JSONArray groupArray = new JSONArray();
            Map<Integer, String> mainTitleMap = new HashMap<Integer, String>();
            for (MarkGroup markGroup : groups) {
                JSONObject group = new JSONObject();
                group.accumulate("groupNumber", markGroup.getNumber());
                List<SubjectiveScore> scores = scoreService.findByStudentIdAndGroupNumber(student.getId(),
                        markGroup.getNumber());
                JSONArray array = new JSONArray();
                for (SubjectiveScore scoreItem : scores) {
                    JSONObject obj = new JSONObject();
                    String mainTitle = mainTitleMap.get(scoreItem.getMainNumber());
                    if (mainTitle == null) {
                        ExamQuestion q = questionService.findByExamAndSubjectAndObjectiveAndMainNumberAndSubNumber(
                                student.getExamId(), student.getSubjectCode(), false, scoreItem.getMainNumber(),
                                scoreItem.getSubNumber());
                        mainTitleMap.put(scoreItem.getMainNumber(), mainTitle);
                        mainTitle = q.getMainTitle();
                    }
                    obj.accumulate("mainTitle", mainTitle);
                    obj.accumulate("questionNumber", scoreItem.getMainNumber() + "-" + scoreItem.getSubNumber());
                    obj.accumulate("score", scoreItem.getScore());
                    array.add(obj);
                }
                group.accumulate("questions", array);
                groupArray.add(group);
            }
            result.accumulate("groups", groupArray);
            // MarkGroup group = groupService.findOne(student.getExamId(),
            // student.getSubjectCode(),
            // library.getGroupNumber());
            List<String> picUrls = fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1,
                    student.getSliceCount());
            // List<MarkTrack> markTracks =
            // markTrackService.findByLibraryId(library.getId());
            // List<MarkSpecialTag> markSpecialTagList =
            // markSpecialTagService.findByLibraryId(library.getId());
            result.accumulate("picUrls", picUrls);
            // result.accumulate("pictureConfig", group.getPictureConfigList());
            // result.accumulate("markTracks", markTracks);
            // result.accumulate("markSpecialTagList", markSpecialTagList);
            result.accumulate("success", true);
        } else {
            result.accumulate("success", false);
        }
        return result;
    }

    @Logging(menu = "评卷复核", type = LogType.UPDATE)
    @RequestMapping("/save")
    @ResponseBody
    public Object save(HttpServletRequest request, @RequestParam Integer studentId) {
        WebUser wu = RequestUtils.getWebUser(request);
        ExamStudent student = studentService.findById(studentId);
        if (student != null && student.getSubjectiveStatus().equals(SubjectiveStatus.MARKED)) {
            studentService.updateSubjectiveStatusAndTimeAndInspectorId(studentId, SubjectiveStatus.INSPECTED,
                    new Date(), wu.getUser().getId());
            return true;
        } else {
            return false;
        }
    }

    @RequestMapping("/clear")
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER })
    public Object clear(HttpServletRequest request, @RequestParam(required = false) Integer studentId) {
        WebUser wu = RequestUtils.getWebUser(request);
        if (studentId != null) {
            releaseTask(studentId);
        } else {
            releaseByUser(wu.getUser().getId());
        }
        return true;
    }

    @Logging(menu = "取消复核", type = LogType.UPDATE)
    @RequestMapping(value = "/cancel", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER })
    public JSONObject cancel(HttpServletRequest request, @RequestParam Integer id) {
        JSONObject obj = new JSONObject();
        ExamStudent student = studentService.findById(id);
        WebUser wu = RequestUtils.getWebUser(request);
        if (student != null && student.getSubjectiveStatus().equals(SubjectiveStatus.INSPECTED)) {
            if (subjectCheck(student.getSubjectCode(), wu)) {
                try {
                    if (lockService.trylock(LockType.STUDENT, student.getId())) {
                        studentService.updateSubjectiveStatusAndTimeAndInspectorId(student.getId(),
                                SubjectiveStatus.MARKED, null, null);
                        obj.accumulate("success", true);
                    } else {
                        obj.accumulate("success", false);
                        obj.accumulate("message", "无法取消复核");
                    }
                } catch (Exception e) {
                    obj.accumulate("success", false);
                    obj.accumulate("message", "取消复核失败");
                    log.error("back inspected error", e);
                } finally {
                    lockService.unlock(LockType.STUDENT, student.getId());
                }
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "没有操作该评卷任务的权限");
            }
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "无法取消复核");
        }
        return obj;
    }

    @Logging(menu = "复核打回", type = LogType.UPDATE)
    @RequestMapping(value = "/back", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER })
    public JSONObject back(HttpServletRequest request, @RequestParam Integer studentId,
            @RequestParam Integer groupNumber) {
        JSONObject obj = new JSONObject();
        ExamStudent student = studentService.findById(studentId);
        WebUser wu = RequestUtils.getWebUser(request);
        if (student != null && student.getSubjectiveStatus().equals(SubjectiveStatus.MARKED)) {
            if (subjectCheck(student.getSubjectCode(), wu) && lockService.trylock(LockType.STUDENT, student.getId())) {
                try {
                    lockService.watch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                    lockService.watch(LockType.GROUP, student.getExamId(), student.getSubjectCode(), groupNumber);
                    if (markService.backStudentByGroup(student, groupNumber, wu.getId())) {
                        obj.accumulate("success", true);
                    } else {
                        obj.accumulate("success", false);
                        obj.accumulate("message", "无法打回");
                    }
                } catch (Exception e) {
                    obj.accumulate("success", false);
                    obj.accumulate("message", "打回失败");
                    log.error("back inspected error", e);
                } finally {
                    lockService.unwatch(LockType.GROUP, student.getExamId(), student.getSubjectCode(), groupNumber);
                    lockService.unwatch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                    lockService.unlock(LockType.STUDENT, student.getId());
                }
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "无法打回");
            }
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "无法取消复核");
        }
        return obj;
    }

    private boolean setCurrent(Integer taskId, Integer userId) {
        Integer value = currentTaskMap.get(taskId);
        if (value == null) {
            synchronized (currentTaskMap) {
                value = currentTaskMap.get(taskId);
                if (value == null) {
                    currentTaskMap.put(taskId, userId);
                    return true;
                } else {
                    return false;
                }
            }
        } else {
            return false;
        }
    }

    private void releaseTask(Integer taskId) {
        synchronized (currentTaskMap) {
            currentTaskMap.remove(taskId);
        }
    }

    private void releaseByUser(Integer userId) {
        Set<Integer> taskIds = new HashSet<>();
        taskIds.addAll(currentTaskMap.keySet());
        synchronized (currentTaskMap) {
            for (Integer taskId : taskIds) {
                Integer value = currentTaskMap.get(taskId);
                if (value != null && value.equals(userId)) {
                    currentTaskMap.remove(taskId);
                }
            }
        }
    }
}
