package cn.com.qmth.stmms.admin.exam;

import cn.com.qmth.stmms.biz.exam.model.*;
import cn.com.qmth.stmms.biz.exam.service.*;
import cn.com.qmth.stmms.biz.file.enums.FormatType;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.*;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;

import java.util.ArrayList;
import java.util.List;

@Controller("libraryController")
@RequestMapping("/admin/exam/library")
public class LibraryController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(LibraryController.class);

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private MarkService markService;

    @Autowired
    private LockService lockService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private ExamService examService;

    @Autowired
    private FileService fileService;

    @Autowired
    private UserService userService;

    @Logging(menu = "评卷任务查询", type = LogType.QUERY)
    @RequestMapping
    public String list(Model model, HttpServletRequest request, MarkLibrarySearchQuery query,
            @RequestParam(required = false) LibraryStatus status) {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (subjectList.isEmpty()) {
            return "redirect:/admin/exam/mark";
        }
        query.setExamId(examId);
        if (status != null) {
            query.addStatus(status);
        }
        if (StringUtils.isEmpty(query.getSubjectCode()) && !subjectList.isEmpty()) {
            query.setSubjectCode(subjectList.get(0).getCode());
        }
        List<MarkGroup> groupList = groupService.findByExamAndSubjectAndStatus(examId, query.getSubjectCode(),
                MarkStatus.FORMAL);
        // if (groupList.isEmpty()) {
        // return "redirect:/admin/exam/mark";
        // }
        if (!groupList.isEmpty()) {
            if (query.getGroupNumber() == 0 && !groupList.isEmpty()) {
                query.setGroupNumber(groupList.get(0).getNumber());
            }
            query.orderById();
            query = libraryService.findByQuery(query);
            for (MarkLibrary library : query.getResult()) {
                if (library.getMarkerId() != null) {
                    Marker marker = markerService.findById(library.getMarkerId());
                    library.setMarkerLoginName(userService.findById(marker.getUserId()).getLoginName());
                }
            }
            for (MarkGroup group : groupList) {
                group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                        group.getSubjectCode(), false, group.getNumber()));
            }
        }
        MarkLibrarySearchQuery query2 = new MarkLibrarySearchQuery();
        query2.setExamId(examId);
        query2.setSubjectCode(query.getSubjectCode());
        query2.setGroupNumber(query.getGroupNumber());
        query2.addStatus(LibraryStatus.MARKED);
        long inspectedCount = libraryService.countByQuery(query2);
        model.addAttribute("query", query);
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        model.addAttribute("groupList", groupList);
        model.addAttribute("statusList", LibraryStatus.getOptionList());
        model.addAttribute("status", status);
        List<Marker> markerList = markerService.findByExamAndSubjectAndGroup(examId, query.getSubjectCode(),
                query.getGroupNumber());
        for (Marker marker : markerList) {
            marker.setLoginName(userService.findById(marker.getUserId()).getLoginName());
        }
        model.addAttribute("markerList", markerList);
        model.addAttribute("inspectedCount", inspectedCount);
        Exam exam = examService.findById(examId);
        model.addAttribute("examType", exam.getType());
        return "modules/exam/libraryList";
    }

    @Logging(menu = "打回评卷任务", type = LogType.UPDATE)
    @RequestMapping(value = "/back", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER })
    public JSONObject back(HttpServletRequest request, @RequestParam Integer id) {
        JSONObject obj = new JSONObject();
        MarkLibrary library = libraryService.findById(id);
        WebUser wu = RequestUtils.getWebUser(request);
        if (library != null) {
            if (subjectCheck(library.getSubjectCode(), RequestUtils.getWebUser(request))) {
                try {
                    lockService.watch(LockType.EXAM_SUBJECT, library.getExamId(), library.getSubjectCode());
                    lockService.watch(LockType.GROUP, library.getExamId(), library.getSubjectCode(),
                            library.getGroupNumber());
                    if ((library.getStatus().equals(LibraryStatus.MARKED) || library.getStatus().equals(
                            LibraryStatus.PROBLEM))
                            && markService.backLibrary(library, wu.getId())) {
                        obj.accumulate("success", true);
                    } else {
                        obj.accumulate("success", false);
                        obj.accumulate("message", "无法打回该评卷任务");
                    }
                } catch (Exception e) {
                    obj.accumulate("success", false);
                    obj.accumulate("message", "打回评卷任务失败");
                    log.error("back library error", e);
                } finally {
                    lockService.unwatch(LockType.GROUP, library.getExamId(), library.getSubjectCode(),
                            library.getGroupNumber());
                    lockService.unwatch(LockType.EXAM_SUBJECT, library.getExamId(), library.getSubjectCode());
                }
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "没有操作该评卷任务的权限");
            }
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "该评卷任务不存在");
        }
        return obj;
    }

    @RequestMapping(value = "/getJson", method = RequestMethod.GET)
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.SCHOOL_VIEWER })
    public String getJson(Model model, HttpServletRequest request, @RequestParam Integer studentId,
            @RequestParam(required = false) Integer groupNumber) {
        int examId = getSessionExamId(request);
        ExamStudent student = studentService.findById(studentId);
        if (groupNumber != null) {
            List<ExamQuestion> questions = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                    student.getSubjectCode(), false, groupNumber);
            List<String> strings = new ArrayList<String>();
            for (ExamQuestion examQuestion : questions) {
                strings.add(examQuestion.getQuestionNumber());
            }
            String questionNumbers = String.join(",", strings);
            model.addAttribute("questionNumbers", questionNumbers);
        }
        model.addAttribute("fileServer", fileService.getFileServer());
        model.addAttribute("answerUrl", fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        model.addAttribute("paperUrl",
                fileService.getPaperUri(student.getExamId(), student.getSubjectCode(), FormatType.JSON));
        return "modules/exam/jsonView";
    }
}
