package cn.com.qmth.stmms.common.controller;

import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import cn.com.qmth.stmms.common.session.service.SessionService;
import cn.com.qmth.stmms.common.utils.EncryptUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;

@Controller
public class LoginController {

    @Autowired
    private UserService userService;

    @Autowired
    private SessionService sessionService;

    @Value("${index.logo}")
    private String indexLogo;

    private static final String LOGIN_TYPE_KEY = "loginType";

    @RequestMapping("/")
    public ModelAndView index(HttpServletRequest request) {
        return new ModelAndView("redirect:/login");
    }

    @RequestMapping(value = "/login", method = RequestMethod.GET)
    public ModelAndView login(HttpServletRequest request) {
        StmmsSession session = RequestUtils.getSession(request);
        session.setParameter(LOGIN_TYPE_KEY, null);
        ModelAndView view = new ModelAndView("modules/sys/login");
        view.addObject("indexLogo", indexLogo);
        view.addObject("showType", "admin-login");
        return view;
    }

    @Logging(menu = "登录", type = LogType.QUERY)
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public ModelAndView login(User user, @RequestParam(required = false) String showType, HttpServletRequest request,
            HttpServletResponse response) {
        StmmsSession session = RequestUtils.getSession(request);
        String loginType = session.getParameter(LOGIN_TYPE_KEY);
        User u = userService.findByLoginName(user.getLoginName());
        ModelAndView modelAndView = new ModelAndView("modules/sys/login");
        modelAndView.addObject("showType", showType);
        modelAndView.addObject("indexLogo", indexLogo);
        if (u == null) {
            modelAndView.addObject("message", "user.login.error.account");
            return modelAndView;
        }
        if (!u.getPassword().equals(EncryptUtils.md5(user.getPassword()))) {
            modelAndView.addObject("message", "user.login.error.password");
            return modelAndView;
        }
        if (!u.isEnable()) {
            modelAndView.addObject("message", "user.login.error.disabled");
            return modelAndView;
        }
        if ("admin-login".equals(showType)) {
            if (u.getRole() == Role.SYS_ADMIN || u.getRole() == Role.SCHOOL_ADMIN || u.getRole() == Role.SUBJECT_HEADER
                    || u.getRole() == Role.SCHOOL_VIEWER) {
                u.setLastLoginTime(new Date());
                u.setLastLoginIp(request.getRemoteAddr());
                u.refreshAccessToken();
                u = userService.save(u);

                session.saveWebUser(new WebUser(u));
                sessionService.put(request, response, session);

                modelAndView = new ModelAndView("redirect:admin/home");
                return modelAndView;
            } else {
                modelAndView.addObject("message", "user.login.error.access");
                return modelAndView;
            }
        } else if ("mark-login".equals(showType) || "mark-login".equals(loginType)) {
            if (u.getLastLoginTime() == null) {
                u.refreshAccessToken();
                userService.save(u);
                session.saveWebUser(new WebUser(u));
                sessionService.put(request, response, session);
                return new ModelAndView("redirect:/mark/reset");
            }
            u.setLastLoginTime(new Date());
            u.setLastLoginIp(request.getRemoteAddr());
            u.refreshAccessToken();
            u = userService.save(u);
            session.saveWebUser(new WebUser(u));
            sessionService.put(request, response, session);
            return new ModelAndView("redirect:/mark/subject-select");
        }
        return modelAndView;
    }

    @RequestMapping(value = "/mark-login", method = RequestMethod.GET)
    public ModelAndView markIndex(HttpServletRequest request) {
        StmmsSession session = RequestUtils.getSession(request);
        session.setParameter(LOGIN_TYPE_KEY, "mark-login");
        ModelAndView view = new ModelAndView("modules/sys/login");
        view.addObject(LOGIN_TYPE_KEY, "mark-login");
        view.addObject("indexLogo", indexLogo);
        return view;
    }

    /**
     * 登出
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping("/logout")
    public ModelAndView adminLogout(HttpServletRequest request, HttpServletResponse response) {
        StmmsSession session = RequestUtils.getSession(request);
        String loginType = session.getParameter(LOGIN_TYPE_KEY);
        RequestUtils.getSession(request).setInvalid(true);
        if ("mark-login".equals(loginType)) {
            return new ModelAndView("redirect:/mark-login");
        } else {
            return new ModelAndView("redirect:/login");
        }
    }

}
